/*
 * dat2wav.c: copy all DAT tracks to disk.
 * Based on playtrack.c from DatLib.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef _WIN32
#include <unistd.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#else
#include "win32.h"
#endif

#include "dataudiolib.h"

#define BUFSIZE 2000

void usage() {
	fprintf(stderr,
		"Usage: dat2wav [-n tracks] [-f tapedevice] <basefilename>\n"
		"   -f Set tape device (default is environment %s or \"%s\".)\n"
		"   -n Read at most this many tracks from current position (no eject)\n"
		"   Files will be called basefilename-01.wav, etc.\n",
#ifdef _WIN32
		"%TAPE%",
#else
		"$TAPE",
#endif
		DEFAULTAUDIOTAPE
		);
	exit(1);
	}

void fputword(unsigned long int x,FILE *w) {
	fputc(x & 0xFF,w); x>>=8;
	fputc(x & 0xFF,w); x>>=8;
	fputc(x & 0xFF,w); x>>=8;
	fputc(x & 0xFF,w); x>>=8;
	}

FILE *newwav(const char *basename,int track,int samplerate) {
	FILE *w;
	char *name,suffix[10]="";
	size_t namelen;
	int i;

	namelen=strlen(basename)+20;
	name=malloc(namelen);
	if (!name) return NULL;
	for(i=0;;i++) {
		if (i>0) sprintf_s(suffix,sizeof(suffix),"-%d",i);
		sprintf_s(name,namelen,"%s-%02d%s.wav",basename,track,suffix);
		w=w32_fopen(name,"rb");
		if (!w) break;
		fclose(w);
		}
	w=w32_fopen(name,"w+b");
	if (!w) {free(name); return NULL;}
	fprintf(w,"RIFF");
	fputword(0,w); /* length of file - 8 */
	fprintf(w,"WAVEfmt ");
	fputword(0x10,w); /* magic */
	fputword(0x00020001,w); /* stereo, magic */
	fputword(samplerate,w);
	fputword(4*samplerate,w); /* bytes/s */
	fputword(0x00100004,w); /* bits/sample, bytes/sample */
	fprintf(w,"data");
	fputword(0,w); /* length of data */
	if (ferror(w)) {fclose(w); return NULL;}
	fprintf(stderr,"New file: %s\n",name);
	free(name);
	return w;
	}

void closewav(FILE *w) {
	long int len;
	do {
		if (ferror(w)) break;
		fseek(w,0,SEEK_END);
		len=ftell(w);
		if (len<0) break;
		fseek(w,0x04,SEEK_SET);
		if (ftell(w)!=0x04) break;
		fputword(len-8,w);
		fseek(w,0x28,SEEK_SET);
		if (ftell(w)!=0x28) break;
		fputword(len-0x2C,w);
		fprintf(stderr,"(length %ld bytes)\n",len);
		} while(0);
	fclose(w);
	}

int main(int argc,char **argv) {
	int	fd;
	char *tape;
	unsigned char buf[BUFSIZE];
	size_t res;
	FILE *w=NULL;
	int	lastsamplerate=0,samplerate,lastprognr,prognr=0;
	struct da_time atime;
	int	absfound=0,noabs=0,noprognr=0,raw=0,absmissing=0,ignore=0;
	int tracksleft=-1;
	const char *basename=NULL;
	u32 errc[2],trackerrc=0;

	tape=NULL;
	argv++; argc--;

	/* process arguments */
	while(argc>0 && (*argv)[0]=='-') {
		char s=(*argv)[1];
printf("%c\n",s);
		argv++; argc--;
		switch(s) {
			case 'f':
				if (argc==0) usage();
				tape=*(argv++); argc--;
				fprintf(stderr,"Tape is %s\n",tape);
				break;
			case 'n':
				if (argc==0) usage();
				tracksleft=atoi(*(argv++)); argc--;
				fprintf(stderr,"Read %d tracks\n",tracksleft);
				break;
			}
		}
	if (argc>0) {
		basename=*(argv++); argc--;
		}
	if (argc>0 || !basename)
		usage();

	/* find tape */
	if (!tape) tape=w32_getenv("TAPE");
	if (!tape) tape=DEFAULTAUDIOTAPE;
	fprintf(stderr,"Opening tape drive: %s\n",tape);
	fd=da_open(tape,O_RDONLY);
	if (fd==-1) {
		fprintf(stderr,"Could not open tape drive (is it attached, is there a tape in?)\n");
		return 1;
		}
	if (tracksleft<=0) /* don't rewind if reading next n tracks */
		da_control(fd,DA_REWIND,NULL);

	for(;;) {
		size_t curleft;

		/* read block */
		for(curleft=0;curleft<BUFSIZE;curleft+=res) {
			res=da_read(fd,buf+curleft,BUFSIZE-(int)curleft);
			if (res==-1) {
				fprintf(stderr,"\nerrno=%d during da_read\n",errno);
				break;
				}
			}
		if (curleft<BUFSIZE) /* failed read */ break;

		/* check for read errors */
		da_control(fd,DA_GET_ERRCOUNTS,errc);

		/* show abs time */
		if (da_control(fd,DA_GET_ABSTIME,&atime)>=0) {
			fprintf(stderr,"\rTrk=%02d AbsTime=%02d:%02d:%02d.%02d (errors=%d) ",
					prognr,
					atime.hour,atime.minute,atime.second,atime.frame,
					errc[0]+errc[1]-trackerrc);
			}

		/* check change-of-program-number */
		lastprognr=prognr;
		if (da_control(fd,DA_GET_PROGNR,&prognr)<0) {
			fprintf(stderr,"errno=%d during da_control DA_GET_PROGNR\n",errno);
			prognr=DA_PNO_INVALID;
			}
		if (prognr==DA_PNO_INVALID) prognr=lastprognr;
		if (prognr==DA_PNO_LEADIN) prognr=0;

		if (lastprognr!=prognr) /* new file: reset */ {
			if (w) {
				closewav(w);
				w=NULL;
				tracksleft--;
				if (tracksleft==0) break;
				}
			trackerrc=errc[0]+errc[1];
			if (da_control(fd,DA_GET_SAMPLERATE,&samplerate)) {
				fprintf(stderr,"errno=%d during da_control DA_GET_SAMPLERATE\n",errno);
				break;
				}
			if (prognr==DA_PNO_LEADOUT) /* finish up */ {
				da_control(fd,DA_REWIND,NULL);
				da_control(fd,DA_EJECT,NULL);
				break;
				}
			w=newwav(basename,prognr,samplerate);
			}

		/* write data to file */
		for(curleft=0;curleft<BUFSIZE;curleft+=res) {
			if (!w) break;
			res=fwrite(buf+curleft,1,BUFSIZE-curleft,w);
			if (res==-1) {
				fprintf(stderr,"write of wav file failed\n");
				break;
				}
			}
		}

	if (w) closewav(w);
	da_close(fd);
	return 0;
	}
